@@ -7,6 +7,7 @@ from ..core.errors import Error
 
 class ConfigError(Error, RuntimeError):
     """Error with parsing or reading a configuration file."""
+
     pass
 
 
@@ -16,8 +17,7 @@ class AuthenticationError(Error, RuntimeError):
 
     def __str__(self):
         if len(self.args) > 0:
-            return "Failed to authenticate with host '{}'.".format(
-                self.args[0])
+            return f"Failed to authenticate with host '{self.args[0]}'."
         else:
             return "Failed to authenticate with host."
 
@@ -25,10 +25,12 @@ class AuthenticationError(Error, RuntimeError):
 # this class is only used by deprecated features
 class ExportError(Error, RuntimeError):
     """Error exporting documents to a mirror."""
+
     pass
 
 
 # this class is only used by deprecated features
 class FetchError(FileNotFoundError):
     """Error in fetching data."""
+
     pass
