@@ -1,52 +1,61 @@
 # Copyright (c) 2017 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
+import getpass
 import logging
 import warnings
-import getpass
 
 from deprecation import deprecated
-from ..version import __version__
+
 from ..core import json
+from ..version import __version__
 from .config import load_config
-from .errors import ConfigError, AuthenticationError
 from .connection import DBClientConnector
-from .crypt import get_crypt_context, SimpleKeyring, get_keyring
+from .crypt import SimpleKeyring, get_crypt_context, get_keyring
+from .errors import AuthenticationError, ConfigError
 
 logger = logging.getLogger(__name__)
 
 
 SESSION_PASSWORD_HASH_CACHE = SimpleKeyring()
-SESSION_USERNAME_CACHE = dict()  # type: ignore
+SESSION_USERNAME_CACHE = {}  # type: ignore
 
 """
 THIS MODULE IS DEPRECATED!
 """
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_default_host(config=None):
     if config is None:
         config = load_config()
     try:
-        return config['General']['default_host']
+        return config["General"]["default_host"]
     except KeyError:
         try:
-            return config['hosts'].keys()[0]
+            return config["hosts"].keys()[0]
         except (KeyError, IndexError):
             raise ConfigError("No hosts specified.")
 
 
 def _get_host_config(hostname, config):
     try:
-        return config['hosts'][hostname]
+        return config["hosts"][hostname]
     except KeyError:
-        raise ConfigError("Host '{}' not configured.".format(hostname))
+        raise ConfigError(f"Host '{hostname}' not configured.")
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_host_config(hostname=None, config=None):
     if config is None:
         config = load_config()
@@ -59,22 +68,26 @@ def _host_id(hostcfg):
     return json.dumps(hostcfg, sort_keys=True)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def make_uri(hostcfg):
-    ret = hostcfg['url']
-    if ret.startswith('mongodb://'):
-        ret = ret[len('mongodb://'):]
-    if 'username' in hostcfg:
-        assert '@' not in ret
-        ret = hostcfg['username'] + '@' + ret
-    return 'mongodb://' + ret
+    ret = hostcfg["url"]
+    if ret.startswith("mongodb://"):
+        ret = ret[len("mongodb://") :]
+    if "username" in hostcfg:
+        assert "@" not in ret
+        ret = hostcfg["username"] + "@" + ret
+    return "mongodb://" + ret
 
 
 def _request_credentials(hostcfg):
-    pwcfg = hostcfg.get('password_config')
-    pw = getpass.getpass("Enter password for {}: ".format(make_uri(hostcfg)))
-    if pwcfg and 'salt' in pwcfg and 'rounds' in pwcfg:
+    pwcfg = hostcfg.get("password_config")
+    pw = getpass.getpass(f"Enter password for {make_uri(hostcfg)}: ")
+    if pwcfg and "salt" in pwcfg and "rounds" in pwcfg:
         logger.debug("Using password configuration for hashing.")
         return get_crypt_context().encrypt(pw, **pwcfg)
     else:
@@ -82,20 +95,19 @@ def _request_credentials(hostcfg):
 
 
 def _get_config_credentials(hostcfg):
-    return hostcfg.get('password')
+    return hostcfg.get("password")
 
 
 def _get_keyring_credentials(hostcfg):
-    pwcfg = hostcfg.get('password_config')
+    pwcfg = hostcfg.get("password_config")
     kr = get_keyring()
     if kr is None:
-        if pwcfg and 'keyring' in pwcfg:
-            warnings.warn(
-                "Password stored in keyring, but keyring is not available!")
-    elif pwcfg and 'keyring' in pwcfg:
-        return kr.get_password('signac', pwcfg['keyring'])
+        if pwcfg and "keyring" in pwcfg:
+            warnings.warn("Password stored in keyring, but keyring is not available!")
+    elif pwcfg and "keyring" in pwcfg:
+        return kr.get_password("signac", pwcfg["keyring"])
     else:
-        return kr.get_password('signac', make_uri(hostcfg))
+        return kr.get_password("signac", make_uri(hostcfg))
 
 
 def _get_cached_credentials(hostcfg, default):
@@ -113,6 +125,7 @@ def _get_stored_credentials(hostcfg):
         if pw is None:
             pw = _get_config_credentials(hostcfg)
         return pw
+
     return _get_cached_credentials(hostcfg, default)
 
 
@@ -124,11 +137,16 @@ def _get_credentials(hostcfg):
             if pw is None:
                 pw = _request_credentials(hostcfg)
         return pw
+
     return _get_cached_credentials(hostcfg, default)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_credentials(hostcfg, ask=True):
     if ask:
         return _get_credentials(hostcfg)
@@ -136,7 +154,7 @@ def get_credentials(hostcfg, ask=True):
         return _get_stored_credentials(hostcfg)
 
 
-def _input(prompt, default=''):
+def _input(prompt, default=""):
     try:
         value = input(prompt)
     except SyntaxError:
@@ -147,37 +165,49 @@ def _input(prompt, default=''):
         return default
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def check_credentials(hostcfg):
     from pymongo.uri_parser import parse_uri
-    auth_m = hostcfg.get('auth_mechanism', 'none')
-    if auth_m == 'SCRAM-SHA-1':
-        uri = hostcfg['url']
+
+    auth_m = hostcfg.get("auth_mechanism", "none")
+    if auth_m == "SCRAM-SHA-1":
+        uri = hostcfg["url"]
         if isinstance(uri, list):
-            uri = ','.join(uri)
-        if 'username' not in hostcfg and not parse_uri(uri)['username']:
+            uri = ",".join(uri)
+        if "username" not in hostcfg and not parse_uri(uri)["username"]:
             username = SESSION_USERNAME_CACHE.get(_host_id(hostcfg))
             if username:
-                hostcfg['username'] = username
+                hostcfg["username"] = username
             else:
-                SESSION_USERNAME_CACHE[_host_id(hostcfg)] = \
-                    hostcfg['username'] = _input(
-                        "Username ({}): ".format(getpass.getuser()),
-                        getpass.getuser())
-        if 'password' not in hostcfg and not parse_uri(uri)['password']:
-            hostcfg['password'] = get_credentials(hostcfg)
+                SESSION_USERNAME_CACHE[_host_id(hostcfg)] = hostcfg[
+                    "username"
+                ] = _input(f"Username ({getpass.getuser()}): ", getpass.getuser())
+        if "password" not in hostcfg and not parse_uri(uri)["password"]:
+            hostcfg["password"] = get_credentials(hostcfg)
     return hostcfg
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_connector(hostcfg, **kwargs):
     return DBClientConnector(hostcfg, **kwargs)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_client(hostcfg, **kwargs):
     connector = get_connector(hostcfg, **kwargs)
     connector.connect()
@@ -185,16 +215,23 @@ def get_client(hostcfg, **kwargs):
     return connector.client
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The host module is deprecated.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The host module is deprecated.",
+)
 def get_database(name, hostname=None, config=None, **kwargs):
     if hostname is None:
         hostname = get_default_host(config)
     if config is None:
         config = load_config()
     hostcfg = check_credentials(get_host_config(hostname, config))
-    logger.debug("Connecting with host config: {}".format(
-        {k: '***' if 'password' in k else v for k, v in hostcfg.items()}))
+    logger.debug(
+        "Connecting with host config: {}".format(
+            {k: "***" if "password" in k else v for k, v in hostcfg.items()}
+        )
+    )
     try:
         client = get_client(hostcfg, **kwargs)
     except Exception as error:
