@@ -5,9 +5,7 @@
 
 import logging
 
-from .configobj.validate import Validator
-from .configobj.validate import VdtValueError
-
+from .configobj.validate import Validator, VdtValueError
 
 logger = logging.getLogger(__name__)
 
@@ -15,9 +13,9 @@ logger = logging.getLogger(__name__)
 def mongodb_uri(value, *args, **kwargs):
     """Return a MongoDB URI."""
     if isinstance(value, list):
-        value = ','.join(value)
-    if not value.startswith('mongodb://'):
-        value = 'mongodb://' + value
+        value = ",".join(value)
+    if not value.startswith("mongodb://"):
+        value = "mongodb://" + value
     try:
         import pymongo
     except ImportError:
@@ -35,10 +33,7 @@ def password(value, *args, **kwargs):  # noqa: D103
 
 
 def get_validator():  # noqa: D103
-    return Validator({
-        'mongodb_uri': mongodb_uri,
-        'password': password,
-    })
+    return Validator({"mongodb_uri": mongodb_uri, "password": password})
 
 
 cfg = """
