@@ -3,8 +3,9 @@
 # This software is licensed under the BSD 3-Clause License.
 """Parse the filter arguments."""
 
+import json
 import sys
-from ..core import json
+from collections.abc import Mapping
 
 
 def _print_err(msg=None):
@@ -35,7 +36,7 @@ def _with_message(query, file):
         Filter arguments.
 
     """
-    print("Interpreted filter arguments as '{}'.".format(json.dumps(query)), file=file)
+    print(f"Interpreted filter arguments as '{json.dumps(query)}'.", file=file)
     return query
 
 
@@ -56,7 +57,7 @@ def _read_index(project, fn_index=None):
 
     """
     if fn_index is not None:
-        _print_err("Reading index from file '{}'...".format(fn_index))
+        _print_err(f"Reading index from file '{fn_index}'...")
         fd = open(fn_index)
         return (json.loads(line) for line in fd)
 
@@ -75,7 +76,7 @@ def _is_json(q):
         True if q starts with "{" and ends with "}".
 
     """
-    return q.strip().startswith('{') and q.strip().endswith('}')
+    return q.strip().startswith("{") and q.strip().endswith("}")
 
 
 def _is_regex(q):
@@ -92,7 +93,7 @@ def _is_regex(q):
         True if q starts with "/" and ends with "/".
 
     """
-    return q.startswith('/') and q.endswith('/')
+    return q.startswith("/") and q.endswith("/")
 
 
 def _parse_json(q):
@@ -112,22 +113,21 @@ def _parse_json(q):
     try:
         return json.loads(q)
     except json.JSONDecodeError:
-        _print_err("Failed to parse query argument. "
-                   "Ensure that '{}' is valid JSON!".format(q))
+        _print_err(f"Failed to parse query argument. Ensure that '{q}' is valid JSON!")
         raise
 
 
 CAST_MAPPING = {
-    'true': True,
-    'false': False,
-    'null': None,
+    "true": True,
+    "false": False,
+    "null": None,
 }
 
 CAST_MAPPING_WARNING = {
-    'True': 'true',
-    'False': 'false',
-    'None': 'null',
-    'none': 'null',
+    "True": "true",
+    "False": "false",
+    "None": "null",
+    "none": "null",
 }
 
 
@@ -147,7 +147,7 @@ def _cast(x):
     """
     try:
         if x in CAST_MAPPING_WARNING:
-            print("Did you mean {}?".format(CAST_MAPPING_WARNING[x]), file=sys.stderr)
+            print(f"Did you mean {CAST_MAPPING_WARNING[x]}?", file=sys.stderr)
         return CAST_MAPPING[x]
     except KeyError:
         try:
@@ -159,7 +159,7 @@ def _cast(x):
                 return x
 
 
-def _parse_simple(key, value=None):
+def _parse_single(key, value=None):
     """Parse simple search syntax.
 
     Parameters
@@ -181,18 +181,42 @@ def _parse_simple(key, value=None):
         If filter arguments have an invalid key.
 
     """
-    if value is None or value == '!':
-        return {key: {'$exists': True}}
+    if value is None or value == "!":
+        return key, {"$exists": True}
     elif _is_json(value):
-        return {key: _parse_json(value)}
+        return key, _parse_json(value)
     elif _is_regex(value):
-        return {key: {'$regex': value[1:-1]}}
+        return key, {"$regex": value[1:-1]}
     elif _is_json(key):
         raise ValueError(
             "Please check your filter arguments. "
-            "Using a JSON expression as a key is not allowed: '{}'.".format(key))
+            "Using a JSON expression as a key is not allowed: '{}'.".format(key)
+        )
     else:
-        return {key: _cast(value)}
+        return key, _cast(value)
+
+
+def parse_simple(tokens):
+    """Parse a set of string tokens into a suitable filter.
+
+    Parameters
+    ----------
+    tokens : Sequence[str]
+        A Sequence of strings composing key-value pairs.
+
+    Yields
+    ------
+    tuple
+        A single key-value pair of input tokenized filter.
+
+    """
+    for i in range(0, len(tokens), 2):
+        key = tokens[i]
+        if i + 1 < len(tokens):
+            value = tokens[i + 1]
+        else:
+            value = None
+        yield _parse_single(key, value)
 
 
 def parse_filter_arg(args, file=sys.stderr):
@@ -217,14 +241,74 @@ def parse_filter_arg(args, file=sys.stderr):
         if _is_json(args[0]):
             return _parse_json(args[0])
         else:
-            return _with_message(_parse_simple(args[0]), file)
+            key, value = _parse_single(args[0])
+            return _with_message({key: value}, file)
     else:
-        q = dict()
-        for i in range(0, len(args), 2):
-            key = args[i]
-            if i+1 < len(args):
-                value = args[i+1]
-            else:
-                value = None
-            q.update(_parse_simple(key, value))
+        q = dict(parse_simple(args))
+
         return _with_message(q, file)
+
+
+def _add_prefix(prefix, filter):
+    """Add desired prefix (e.g. 'sp.' or 'doc.') to a (possibly nested) filter."""
+    if filter:
+        for key, value in filter.items():
+            if key in ("$and", "$or"):
+                if isinstance(value, list) or isinstance(value, tuple):
+                    yield key, [dict(_add_prefix(prefix, item)) for item in value]
+                else:
+                    raise ValueError(
+                        "The argument to a logical operator must be a list or a tuple!"
+                    )
+            elif "." in key and key.split(".", 1)[0] in ("sp", "doc"):
+                yield key, value
+            elif key in ("sp", "doc"):
+                yield key, value
+            else:
+                yield prefix + key, value
+
+
+def _root_keys(filter):
+    for key, value in filter.items():
+        if key in ("$and", "$or"):
+            assert isinstance(value, (list, tuple))
+            for item in value:
+                for key in _root_keys(item):
+                    yield key
+        elif "." in key:
+            yield key.split(".", 1)[0]
+        else:
+            yield key
+
+
+def parse_filter(filter):
+    """Parse a provided sequence of filters.
+
+    Parameters
+    ----------
+    filter : Sequence, Mapping, or str
+        A set of key, value tuples corresponding to a single filter. This
+        filter may itself be a compound filter containing and/or statements. The
+        filter may be provided as a sequence of tuples, a mapping-like object,
+        or a string. In the last case, the string will be parsed to generate a
+        valid set of filters.
+
+    Yields
+    ------
+    tuple
+        A key value pair to be used as a filter.
+
+    """
+    if isinstance(filter, str):
+        yield from parse_simple(filter.split())
+    elif isinstance(filter, Mapping):
+        yield from filter.items()
+    else:
+        try:
+            yield from filter
+        except TypeError:
+            # This type was not iterable.
+            raise ValueError(
+                f"Invalid filter type {type(filter)}. The filter must "
+                "be a Sequence, Mapping, or str."
+            )
