@@ -3,45 +3,41 @@
 # This software is licensed under the BSD 3-Clause License.
 """Provides features for importing and exporting data."""
 
+import errno
+import json
+import logging
 import os
 import re
-import errno
 import shutil
-import zipfile
 import tarfile
-from zipfile import ZipFile, ZIP_DEFLATED
+import zipfile
 from collections import OrderedDict
-from contextlib import contextmanager, closing
+from contextlib import closing, contextmanager
 from string import Formatter
 from tempfile import TemporaryDirectory
+from zipfile import ZIP_DEFLATED, ZipFile
 
-from ..core import json
-from .errors import StatepointParsingError
-from .errors import DestinationExistsError
-from .utility import _mkdir_p
-from .utility import _dotted_dict_to_nested_dicts
-
-import logging
-
+from .errors import DestinationExistsError, StatepointParsingError
+from .utility import _dotted_dict_to_nested_dicts, _mkdir_p
 
 logger = logging.getLogger(__name__)
 
 
-_DOT_MAGIC_WORD = '__DOT__'
+_DOT_MAGIC_WORD = "__DOT__"
 
 
 RE_TYPES = {
-    'str': r'\w+',
-    'int': r'[+-]?[0-9]+',
-    'float': r'[+-]?([0-9]*[\.])?[0-9]+',
-    'bool': r'\w+',
+    "str": r"\w+",
+    "int": r"[+-]?[0-9]+",
+    "float": r"[+-]?([0-9]*[\.])?[0-9]+",
+    "bool": r"\w+",
 }
 
 
 #  ### Export related  ###
 
 
-def _make_schema_based_path_function(jobs, exclude_keys=None, delimiter_nested='.'):
+def _make_schema_based_path_function(jobs, exclude_keys=None, delimiter_nested="."):
     """Generate schema-based paths as a function of the given jobs.
 
     Parameters
@@ -60,18 +56,19 @@ def _make_schema_based_path_function(jobs, exclude_keys=None, delimiter_nested='
 
     """
     from .schema import _build_job_statepoint_index
+
     if len(jobs) <= 1:
         # The lambda must (optionally) take a format spec argument to match the
         # signature of the path function below.
-        return lambda job, sep=None: ''
+        return lambda job, sep=None: ""
 
-    index = [{'_id': job._id, 'statepoint': job.sp()} for job in jobs]
+    index = [{"_id": job.id, "sp": job.sp()} for job in jobs]
     jsi = _build_job_statepoint_index(exclude_const=True, index=index)
     sp_index = OrderedDict(jsi)
 
-    paths = dict()
+    paths = {}
     for key_tokens, values in sp_index.items():
-        key = key_tokens.replace('.', delimiter_nested)
+        key = key_tokens.replace(".", delimiter_nested)
         if exclude_keys and key in exclude_keys:
             continue
         for value, group in values.items():
@@ -103,14 +100,16 @@ def _make_schema_based_path_function(jobs, exclude_keys=None, delimiter_nested='
         """
         try:
             if sep:
-                return os.path.normpath(sep.join(paths[job._id]))
+                return os.path.normpath(sep.join(paths[job.id]))
             else:
-                return os.path.normpath(os.path.join(*paths[job._id]))
+                return os.path.normpath(os.path.join(*paths[job.id]))
         except KeyError:
             raise RuntimeError(
                 "Unable to determine path for job '{}'.\nThis is usually caused by a "
                 "heterogeneous schema, where some keys are only present in some jobs. "
-                "Try providing a custom path.".format(job))
+                "Try providing a custom path.".format(job)
+            )
+
     return path
 
 
@@ -127,11 +126,9 @@ class _AutoPathFormatter(Formatter):
         ----------
         value :
             The value to be formatted.
-
         format_spec :
             The format specification.
 
-
         Returns
         -------
         str
@@ -139,14 +136,16 @@ class _AutoPathFormatter(Formatter):
 
         """
         from .job import Job
+
         if isinstance(value, Job):
             return self.paths(value, format_spec)
         else:
-            return super(_AutoPathFormatter, self).format_field(value, format_spec)
+            return super().format_field(value, format_spec)
 
 
 class _SchemaPathEvaluationError(RuntimeError):
     """Raised for errors in schema path evaluation."""
+
     pass
 
 
@@ -192,7 +191,7 @@ def _make_path_function(jobs, path):
                 Job id.
 
             """
-            return str(job.get_id())
+            return str(job.id)
 
     elif isinstance(path, str):
         # Detect keys that are already provided as part of the path specifier and
@@ -219,27 +218,33 @@ def _make_path_function(jobs, path):
             """
             try:
                 try:
-                    ret = path.format(job=job, **job.sp)
+                    ret = path.format(job=job, **job.statepoint)
                 except TypeError as error:
-                    if str(error) == "format() got multiple values for keyword argument 'job'":
+                    if (
+                        str(error)
+                        == "format() got multiple values for keyword argument 'job'"
+                    ):
                         try:
                             ret = path.format(job=job)
                         except KeyError:
                             raise _SchemaPathEvaluationError(
                                 "You must use fully qualified fields for this path, because the "
                                 "state point contains a key called 'job', e.g.: '{job.sp.job}' "
-                                "instead of '{job}'.")
+                                "instead of '{job}'."
+                            )
                     ret = path.format(job=job)
                 return _AutoPathFormatter(paths).format(ret, auto=job)
             except AttributeError as error:
                 raise _SchemaPathEvaluationError(error)
             except KeyError as error:
-                raise _SchemaPathEvaluationError("Unknown key: {}".format(error))
+                raise _SchemaPathEvaluationError(f"Unknown key: {error}")
             except Exception as error:
                 raise _SchemaPathEvaluationError(error)
+
     else:
         raise ValueError(
-            "The path argument must either be `None`, `False`, or of type `str`.")
+            "The path argument must either be `None`, `False`, or of type `str`."
+        )
 
     return path_function
 
@@ -262,7 +267,8 @@ def _check_directory_structure_validity(paths):
     for dst in paths:
         if dst in check:
             raise RuntimeError(
-                "The path '{}' is both a leaf and node in the path structure.".format(dst))
+                f"The path '{dst}' is both a leaf and node in the path structure."
+            )
         tokens = dst.split(os.path.sep)
         for i in range(1, len(tokens)):
             check.add(os.path.sep.join(tokens[:i]))
@@ -412,7 +418,8 @@ def export_to_zipfile(jobs, zipfile, path=None):
             for fn in filenames:
                 zipfile.write(
                     filename=os.path.join(root, fn),
-                    arcname=os.path.join(dst, os.path.relpath(root, src), fn))
+                    arcname=os.path.join(dst, os.path.relpath(root, src), fn),
+                )
 
     return _export_jobs(jobs=jobs, path=path, copytree=copytree_to_zip)
 
@@ -452,39 +459,40 @@ def export_jobs(jobs, target, path=None, copytree=None):
 
     """
     if copytree is not None:
-        if not (isinstance(target, str) and os.path.splitext(target)[1] == ''):
+        if not (isinstance(target, str) and os.path.splitext(target)[1] == ""):
             raise ValueError(
                 "The copytree argument can only be used in combination "
-                "with directories as targets.")
+                "with directories as targets."
+            )
 
     if isinstance(target, str):
         ext = os.path.splitext(target)[1]
-        if ext == '':  # target is directory
+        if ext == "":  # target is directory
             yield from export_to_directory(
-                jobs=jobs, target=target, path=path, copytree=copytree)
-        elif ext == '.zip':     # target is zipfile
-            with ZipFile(target, mode='w', compression=ZIP_DEFLATED) as zipfile:
+                jobs=jobs, target=target, path=path, copytree=copytree
+            )
+        elif ext == ".zip":  # target is zipfile
+            with ZipFile(target, mode="w", compression=ZIP_DEFLATED) as zipfile:
                 yield from export_to_zipfile(jobs=jobs, zipfile=zipfile, path=path)
-        elif ext == '.tar':     # target is uncompressed tarball
-            with tarfile.open(name=target, mode='a') as file:
+        elif ext == ".tar":  # target is uncompressed tarball
+            with tarfile.open(name=target, mode="a") as file:
                 yield from export_to_tarfile(jobs=jobs, tarfile=file, path=path)
-        elif ext in ('.gz', '.bz2', '.xz'):    # target is compressed tarball
-            with tarfile.open(name=target, mode='w:' + ext[1:]) as file:
+        elif ext in (".gz", ".bz2", ".xz"):  # target is compressed tarball
+            with tarfile.open(name=target, mode="w:" + ext[1:]) as file:
                 yield from export_to_tarfile(jobs=jobs, tarfile=file, path=path)
         else:
-            raise TypeError("Unknown extension '{}'.".format(ext))
+            raise TypeError(f"Unknown extension '{ext}'.")
     elif isinstance(target, ZipFile):
-        for src_dst in export_to_zipfile(jobs=jobs, zipfile=target, path=path):
-            yield src_dst
+        yield from export_to_zipfile(jobs=jobs, zipfile=target, path=path)
     elif isinstance(target, tarfile.TarFile):
-        for src_dst in export_to_tarfile(jobs=jobs, tarfile=target, path=path):
-            yield src_dst
+        yield from export_to_tarfile(jobs=jobs, tarfile=target, path=path)
     else:
         raise TypeError("Unknown target type", target)
 
 
 # ### Import related ###
 
+
 def _convert_bool(value):
     """Convert a boolean value encoded as string to corresponding bool.
 
@@ -499,10 +507,9 @@ def _convert_bool(value):
         Boolean interpreted from string.
 
     """
-    return {
-        'true': True,   '1': True,
-        'false': False, '0': False,
-    }.get(value.lower(), bool(value))
+    return {"true": True, "1": True, "false": False, "0": False}.get(
+        value.lower(), bool(value)
+    )
 
 
 def _convert_schema_path_to_regex(schema_path):
@@ -533,34 +540,34 @@ def _convert_schema_path_to_regex(schema_path):
     """
     # First, replace escaped backslashes with double-escaped backslashes.
     # This is needed for compatibility with Windows, which uses backslashes.
-    schema_path = re.sub(r'\\', r'\\\\', schema_path)
+    schema_path = re.sub(r"\\", r"\\\\", schema_path)
 
     # The regular expression below is used to identify the {value:type} specifications
     # in the schema path.
-    re_key_type_field = r'\{(?P<key>[\.\w]+)(?::(?P<type>[a-z]+))?\}'
-    schema_regex = ''    # the return value
-    types = dict()       # maps values to their designated types
+    re_key_type_field = r"\{(?P<key>[\.\w]+)(?::(?P<type>[a-z]+))?\}"
+    schema_regex = ""  # the return value
+    types = {}  # maps values to their designated types
     index = 0
     while True:
         m = re.search(re_key_type_field, schema_path[index:])
         if m:
-            key = m.groupdict()['key'].replace('.', _DOT_MAGIC_WORD)
-            types[key] = m.groupdict()['type'] or 'str'
+            key = m.groupdict()["key"].replace(".", _DOT_MAGIC_WORD)
+            types[key] = m.groupdict()["type"] or "str"
             start, stop = m.span()
-            schema_regex += schema_path[index:index+start].replace('.', r'\.')
-            schema_regex += r'(?P<{}>{})'.format(key, RE_TYPES[types[key]])
+            schema_regex += schema_path[index : index + start].replace(".", r"\.")
+            schema_regex += fr"(?P<{key}>{RE_TYPES[types[key]]})"
             index += stop
             continue
         break
-    schema_regex += '$'
+    schema_regex += "$"
 
     for key in types:
-        if types[key] in ('int', 'float', 'str'):
+        if types[key] in ("int", "float", "str"):
             types[key] = eval(types[key])
-        elif types[key] == 'bool':
+        elif types[key] == "bool":
             types[key] = _convert_bool
         else:
-            raise ValueError("Invalid type '{}'.".format(types[key]))
+            raise ValueError(f"Invalid type '{types[key]}'.")
     return schema_regex, types
 
 
@@ -596,11 +603,13 @@ def _make_path_based_schema_function(schema_path):
         """
         match = re.match(schema_regex, os.path.normpath(path))
         if match:
-            sp = match.groupdict()
+            statepoint = match.groupdict()
             for key in types:
-                if key in sp:
-                    sp[key] = types[key](sp[key])
-            return _dotted_dict_to_nested_dicts(sp, delimiter_nested=_DOT_MAGIC_WORD)
+                if key in statepoint:
+                    statepoint[key] = types[key](statepoint[key])
+            return _dotted_dict_to_nested_dicts(
+                statepoint, delimiter_nested=_DOT_MAGIC_WORD
+            )
 
     return parse_path
 
@@ -621,6 +630,7 @@ def _with_consistency_check(schema_function, read_sp_manifest_file):
         Schema checking function.
 
     """
+
     def _check(path):
         """Check if the schema-detected state point matches the manifest file.
 
@@ -636,7 +646,7 @@ def _with_consistency_check(schema_function, read_sp_manifest_file):
 
         Raises
         ------
-        StatepointParsingError
+        :class:`~signac.errors.StatepointParsingError`
             If identified state point conflicts with state point in job manifest file.
 
         """
@@ -647,8 +657,10 @@ def _with_consistency_check(schema_function, read_sp_manifest_file):
             sp_default = read_sp_manifest_file(path)
             if sp and sp_default and sp_default != sp:
                 raise StatepointParsingError(
-                    "Identified state point conflicts with state point in job manifest file!")
+                    "Identified state point conflicts with state point in job manifest file!"
+                )
             return sp
+
     return _check
 
 
@@ -666,6 +678,7 @@ def _parse_workspaces(fn_manifest):
         Function to parse a manifest, given a path.
 
     """
+
     def _parse_workspace(path):
         """Parse a manifest, given a path.
 
@@ -681,9 +694,9 @@ def _parse_workspaces(fn_manifest):
 
         """
         try:
-            with open(os.path.join(path, fn_manifest), 'rb') as file:
+            with open(os.path.join(path, fn_manifest), "rb") as file:
                 return json.loads(file.read().decode())
-        except (IOError, OSError) as error:
+        except OSError as error:
             if error.errno != errno.ENOENT:
                 raise error
 
@@ -716,13 +729,13 @@ def _crawl_directory_data_space(root, project, schema_function):
     for path, dirs, _ in os.walk(root):
         sp = schema_function(path)
         if sp is not None:
-            del dirs[:]         # skip sub-directories
+            del dirs[:]  # skip sub-directories
             job = project.open_job(sp)
             dst = job.workspace()
             if os.path.realpath(path) == os.path.realpath(dst):
-                continue     # skip (already part of the data space)
+                continue  # skip (already part of the data space)
             elif os.path.realpath(path).startswith(workspace_real_path):
-                continue     # skip (part of the project's workspace)
+                continue  # skip (part of the project's workspace)
             yield path, job
 
 
@@ -748,16 +761,16 @@ def _copy_to_job_workspace(src, job, copytree):
     dst = job.workspace()
     try:
         copytree(src, dst)
-    except (IOError, OSError) as error:
-        if error.errno in (errno.ENOTEMPTY, errno.EEXIST):
+    except OSError as error:
+        if error.errno in (errno.EEXIST, errno.ENOTEMPTY, errno.EACCES):
             raise DestinationExistsError(job)
         raise
     else:
-        job._init()
+        job.init()
     return dst
 
 
-class _CopyFromDirectoryExecutor(object):
+class _CopyFromDirectoryExecutor:
     """Copy the source to job's workspace when the source is a directory.
 
     Parameters
@@ -768,6 +781,7 @@ class _CopyFromDirectoryExecutor(object):
         An instance of :class:`~signac.contrib.job.Job`.
 
     """
+
     def __init__(self, src, job):
         self.src = src
         self.job = job
@@ -803,7 +817,7 @@ def _analyze_directory_for_import(root, project, schema):
     ------
     TypeError
         If the schema given is not None, callable, or a string.
-    StatepointParsingError
+    :class:`~signac.errors.StatepointParsingError`
         If the jobs identified with the given schema function are not unique.
 
     """
@@ -817,7 +831,8 @@ def _analyze_directory_for_import(root, project, schema):
         if not schema.startswith(root):
             schema = os.path.normpath(os.path.join(root, schema))
         schema_function = _with_consistency_check(
-            _make_path_based_schema_function(schema), read_sp_manifest_file)
+            _make_path_based_schema_function(schema), read_sp_manifest_file
+        )
     else:
         raise TypeError("The schema variable must be None, callable, or a string.")
 
@@ -826,14 +841,15 @@ def _analyze_directory_for_import(root, project, schema):
     for src, job in _crawl_directory_data_space(root, project, schema_function):
         if job in jobs:
             raise StatepointParsingError(
-                "The jobs identified with the given schema function are not unique!")
+                "The jobs identified with the given schema function are not unique!"
+            )
         else:
             jobs.add(job)
             copy_executor = _CopyFromDirectoryExecutor(src, job)
             yield src, copy_executor
 
 
-class _CopyFromZipFileExecutor(object):
+class _CopyFromZipFileExecutor:
     """Copy the source to job's workspace when the source is a zipfile.
 
     Parameters
@@ -848,6 +864,7 @@ class _CopyFromZipFileExecutor(object):
         File names to copy.
 
     """
+
     def __init__(self, zipfile, root, job, names):
         self.zipfile = zipfile
         self.root = root
@@ -860,12 +877,12 @@ class _CopyFromZipFileExecutor(object):
         for name in self.names:
             fn_dst = self.job.fn(os.path.relpath(name, self.root))
             _mkdir_p(os.path.dirname(fn_dst))
-            with open(fn_dst, 'wb') as dst:
+            with open(fn_dst, "wb") as dst:
                 dst.write(self.zipfile.read(name))
         return self.job.workspace()
 
     def __str__(self):
-        return "{}({} -> {})".format(type(self).__name__, self.root, self.job)
+        return f"{type(self).__name__}({self.root} -> {self.job})"
 
 
 def _analyze_zipfile_for_import(zipfile, project, schema):
@@ -893,9 +910,9 @@ def _analyze_zipfile_for_import(zipfile, project, schema):
     ------
     TypeError
         If the schema provided is not None, callable, or a string.
-    DestinationExistsError
+    :class:`~signac.errors.DestinationExistsError`
         If a job is already initialized.
-    StatepointParsingError
+    :class:`~signac.errors.StatepointParsingError`
         If the jobs identified with the given schema function are not unique.
 
     """
@@ -916,7 +933,7 @@ def _analyze_zipfile_for_import(zipfile, project, schema):
 
         """
         # Must use forward slashes, not os.path.sep.
-        fn_manifest = path + '/' + project.Job.FN_MANIFEST
+        fn_manifest = path + "/" + project.Job.FN_MANIFEST
         if fn_manifest in names:
             return json.loads(zipfile.read(fn_manifest).decode())
 
@@ -926,11 +943,12 @@ def _analyze_zipfile_for_import(zipfile, project, schema):
         schema_function = _with_consistency_check(schema, read_sp_manifest_file)
     elif isinstance(schema, str):
         schema_function = _with_consistency_check(
-            _make_path_based_schema_function(schema), read_sp_manifest_file)
+            _make_path_based_schema_function(schema), read_sp_manifest_file
+        )
     else:
         raise TypeError("The schema variable must be None, callable, or a string.")
 
-    mappings = dict()
+    mappings = {}
     skip_subdirs = set()
 
     dirs = {os.path.dirname(name) for name in names}
@@ -954,7 +972,8 @@ def _analyze_zipfile_for_import(zipfile, project, schema):
     # Check uniqueness
     if len(set(mappings.values())) != len(mappings):
         raise StatepointParsingError(
-            "The jobs identified with the given schema function are not unique!")
+            "The jobs identified with the given schema function are not unique!"
+        )
 
     for src, job in mappings.items():
         _names = [name for name in names if name.startswith(src)]
@@ -962,7 +981,7 @@ def _analyze_zipfile_for_import(zipfile, project, schema):
         yield src, copy_executor
 
 
-class _CopyFromTarFileExecutor(object):
+class _CopyFromTarFileExecutor:
     """Copy the source to job's workspace when the source is a tarfile.
 
     Parameters
@@ -1005,8 +1024,8 @@ def _tarfile_path_join(path, fn):
         Path with normalized forward slashes.
 
     """
-    path = path.rstrip('/')
-    return path + '/' + fn
+    path = path.rstrip("/")
+    return path + "/" + fn
 
 
 def _analyze_tarfile_for_import(tarfile, project, schema, tmpdir):
@@ -1036,9 +1055,9 @@ def _analyze_tarfile_for_import(tarfile, project, schema, tmpdir):
     ------
     TypeError
         If the schema given is not None, callable, or a string.
-    DestinationExistsError
+    :class:`~signac.errors.DestinationExistsError`
         If a job is already initialized.
-    StatepointParsingError
+    :class:`~signac.errors.StatepointParsingError`
         If the jobs identified with the given schema function are not unique.
     AssertionError
         If ``tmpdir`` given is not a directory.
@@ -1073,16 +1092,19 @@ def _analyze_tarfile_for_import(tarfile, project, schema, tmpdir):
         schema_function = _with_consistency_check(schema, read_sp_manifest_file)
     elif isinstance(schema, str):
         schema_function = _with_consistency_check(
-            _make_path_based_schema_function(schema), read_sp_manifest_file)
+            _make_path_based_schema_function(schema), read_sp_manifest_file
+        )
     else:
         raise TypeError("The schema variable must be None, callable, or a string.")
 
-    mappings = dict()
+    mappings = {}
     skip_subdirs = set()
 
     dirs = [member.name for member in tarfile.getmembers() if member.isdir()]
     for name in sorted(dirs):
-        if os.path.dirname(name) in skip_subdirs:   # skip all sub-dirs of identified dirs
+        if (
+            os.path.dirname(name) in skip_subdirs
+        ):  # skip all sub-dirs of identified dirs
             skip_subdirs.add(name)
             continue
 
@@ -1097,7 +1119,8 @@ def _analyze_tarfile_for_import(tarfile, project, schema, tmpdir):
     # Check uniqueness
     if len(set(mappings.values())) != len(mappings):
         raise StatepointParsingError(
-            "The jobs identified with the given schema function are not unique!")
+            "The jobs identified with the given schema function are not unique!"
+        )
 
     tarfile.extractall(path=tmpdir)
     for path, job in mappings.items():
@@ -1147,11 +1170,11 @@ def _prepare_import_into_project(origin, project, schema=None):
                 with tarfile.open(origin) as file:
                     yield _analyze_tarfile_for_import(file, project, schema, tmpdir)
         else:
-            raise RuntimeError("Unknown file type: '{}'.".format(origin))
+            raise RuntimeError(f"Unknown file type: '{origin}'.")
     elif os.path.isdir(origin):
         yield _analyze_directory_for_import(root=origin, project=project, schema=schema)
     else:
-        raise ValueError("Unable to import from '{}'. Does the origin exist?".format(origin))
+        raise ValueError(f"Unable to import from '{origin}'. Does the origin exist?")
 
 
 def import_into_project(origin, project, schema=None, copytree=None):
