@@ -3,20 +3,20 @@
 # This software is licensed under the BSD 3-Clause License.
 """Utilities for signac."""
 
-import logging
-import sys
-import os
-import getpass
 import argparse
-import errno
-import zipfile
+import getpass
+import logging
+import os
+import sys
 import tarfile
-from time import time
-from datetime import timedelta
-from contextlib import contextmanager
-from deprecation import deprecated
-from tempfile import TemporaryDirectory
+import zipfile
 from collections.abc import Mapping
+from contextlib import contextmanager
+from datetime import timedelta
+from tempfile import TemporaryDirectory
+from time import time
+
+from deprecation import deprecated
 
 from ..version import __version__
 
@@ -64,16 +64,15 @@ def query_yes_no(question, default="yes"):
     while True:
         sys.stdout.write(question + prompt)
         choice = input().lower()
-        if default is not None and choice == '':
+        if default is not None and choice == "":
             return valid[default]
         elif choice in valid:
             return valid[choice]
         else:
-            sys.stdout.write("Please respond with 'yes' or 'no' "
-                             "(or 'y' or 'n').\n")
+            sys.stdout.write("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")
 
 
-def prompt_password(prompt='Password: '):
+def prompt_password(prompt="Password: "):
     """Prompt password for user.
 
     Parameters
@@ -108,15 +107,20 @@ def add_verbosity_argument(parser, default=0):
 
     """
     parser.add_argument(
-        '-v', '--verbosity',
+        "-v",
+        "--verbosity",
         help="Set level of verbosity.",
-        action='count',
+        action="count",
         default=default,
     )
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="This function is obsolete.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="This function is obsolete.",
+)
 def add_verbosity_action_argument(parser, default=0):
     """Add a verbosity argument to parser.
 
@@ -135,16 +139,20 @@ def add_verbosity_action_argument(parser, default=0):
 
     """
     parser.add_argument(
-        '-v',
+        "-v",
         default=0,
-        nargs='?',
+        nargs="?",
         action=VerbosityLoggingConfigAction,
-        dest='verbosity',
+        dest="verbosity",
     )
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="This function is obsolete.")
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="This function is obsolete.",
+)
 def set_verbosity_level(verbosity, default=None, increment=10):
     """Set the verbosity level as a function of an integer level.
 
@@ -160,50 +168,65 @@ def set_verbosity_level(verbosity, default=None, increment=10):
     """
     if default is None:
         default = logging.ERROR
-    logging.basicConfig(
-        level=default - increment * verbosity)
+    logging.basicConfig(level=default - increment * verbosity)
 
 
 # this class is deprecated
-class VerbosityAction(argparse.Action): # noqa: D101, E261
-    @deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-                details="This class is obsolete")
-    def __call__(self, parser, args, values, option_string=None): # noqa: D102, E261
+class VerbosityAction(argparse.Action):  # noqa: D101, E261
+    @deprecated(
+        deprecated_in="1.3",
+        removed_in="2.0",
+        current_version=__version__,
+        details="This class is obsolete",
+    )
+    def __call__(self, parser, args, values, option_string=None):  # noqa: D102, E261
         if values is None:
-            values = '1'
+            values = "1"
         try:
             values = int(values)
         except ValueError:
-            values = values.count('v') + 1
+            values = values.count("v") + 1
         setattr(args, self.dest, values)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The VerbosityLoggingConfigAction class is obsolete.")
-class VerbosityLoggingConfigAction(VerbosityAction): # noqa: D101, E261
-    def __call__(self, parser, args, values, option_string=None): # noqa:D102, E261
-        super(VerbosityLoggingConfigAction, self).__call__(
-            parser, args, values, option_string)
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The VerbosityLoggingConfigAction class is obsolete.",
+)
+class VerbosityLoggingConfigAction(VerbosityAction):  # noqa: D101, E261
+    def __call__(self, parser, args, values, option_string=None):  # noqa:D102, E261
+        super().__call__(parser, args, values, option_string)
         v_level = getattr(args, self.dest)
         set_verbosity_level(v_level)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The EmptyIsTrue class is obsolete.")
-class EmptyIsTrue(argparse.Action): # noqa: D101, E261
-    def __call__(self, parser, namespace, values, option_string=None): # noqa: D102, E261
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The EmptyIsTrue class is obsolete.",
+)
+class EmptyIsTrue(argparse.Action):  # noqa: D101, E261
+    def __call__(
+        self, parser, namespace, values, option_string=None
+    ):  # noqa: D102, E261
         if values is None:
             values = True
         setattr(namespace, self.dest, values)
 
 
-@deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
-            details="The SmartFormatter class is obsolete.")
-class SmartFormatter(argparse.HelpFormatter): # noqa: D101, E261
-
+@deprecated(
+    deprecated_in="1.3",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The SmartFormatter class is obsolete.",
+)
+class SmartFormatter(argparse.HelpFormatter):  # noqa: D101, E261
     def _split_lines(self, text, width):
 
-        if text.startswith('R|'):
+        if text.startswith("R|"):
             return text[2:].splitlines()
         return argparse.HelpFormatter._split_lines(self, text, width)
 
@@ -234,12 +257,11 @@ def walkdepth(path, depth=0):
 
     """
     if depth == 0:
-        for p in os.walk(path):
-            yield p
+        yield from os.walk(path)
     elif depth > 0:
         path = path.rstrip(os.path.sep)
         if not os.path.isdir(path):
-            raise OSError("Not a directory: '{}'.".format(path))
+            raise OSError(f"Not a directory: '{path}'.")
         num_sep = path.count(os.path.sep)
         for root, dirs, files in os.walk(path):
             yield root, dirs, files
@@ -251,7 +273,7 @@ def walkdepth(path, depth=0):
 
 
 def _mkdir_p(path):
-    """Make a new directory.
+    """Make a new directory, or do nothing if the directory already exists.
 
     Parameters
     ----------
@@ -259,14 +281,15 @@ def _mkdir_p(path):
         New directory name.
 
     """
-    try:
-        os.makedirs(path)
-    except OSError as error:
-        if not (error.errno == errno.EEXIST and os.path.isdir(path)):
-            raise
+    # Performance: `isdir` is fast and eliminates the rest of os.makedirs
+    # if the path already exists. Typically this function is called in cases
+    # where the path already exists. If the path is a file, this check returns
+    # False and allows os.makedirs to raise FileExistsError as usual.
+    if not os.path.isdir(path):
+        os.makedirs(path, exist_ok=True)
 
 
-def split_and_print_progress(iterable, num_chunks=10, write=None, desc='Progress: '):
+def split_and_print_progress(iterable, num_chunks=10, write=None, desc="Progress: "):
     """Split the progress and prints it.
 
     Parameters
@@ -298,21 +321,21 @@ def split_and_print_progress(iterable, num_chunks=10, write=None, desc='Progress
         len_chunk = int(N / num_chunks)
         intervals = []
         show_est = False
-        for i in range(num_chunks-1):
+        for i in range(num_chunks - 1):
             if i:
-                msg = "{}{:3.0f}%".format(desc, 100 * i / num_chunks)
+                msg = f"{desc}{100 * i / num_chunks:3.0f}%"
                 if intervals:
                     mean_interval = sum(intervals) / len(intervals)
                     est_remaining = int(mean_interval * (num_chunks - i))
                     if est_remaining > 10 or show_est:
                         show_est = True
-                        msg += " (ETR: {}h)".format(timedelta(seconds=est_remaining))
+                        msg += f" (ETR: {timedelta(seconds=est_remaining)}h)"
                 write(msg)
             start = time()
-            yield iterable[i * len_chunk:(i+1) * len_chunk]
+            yield iterable[i * len_chunk : (i + 1) * len_chunk]
             intervals.append(time() - start)
-        yield iterable[(i+1) * len_chunk:]
-        write("{}100%".format(desc))
+        yield iterable[(i + 1) * len_chunk :]
+        write(f"{desc}100%")
     else:
         yield iterable
 
@@ -347,10 +370,10 @@ def _extract(filename):
                 file.extractall(path=tmpdir)
                 yield tmpdir
         else:
-            raise RuntimeError("Unknown file type: '{}'.".format(filename))
+            raise RuntimeError(f"Unknown file type: '{filename}'.")
 
 
-def _dotted_dict_to_nested_dicts(dotted_dict, delimiter_nested='.'):
+def _dotted_dict_to_nested_dicts(dotted_dict, delimiter_nested="."):
     """Convert dotted keys in the state point dict to a nested dict.
 
     Parameters
@@ -366,13 +389,13 @@ def _dotted_dict_to_nested_dicts(dotted_dict, delimiter_nested='.'):
         A mapping instance with nested dicts, e.g. {'a': {'b': 'c'}}.
 
     """
-    nested_dict = dict()
+    nested_dict = {}
     for key, value in dotted_dict.items():
         tokens = key.split(delimiter_nested)
         if len(tokens) > 1:
-            tmp = nested_dict.setdefault(tokens[0], dict())
+            tmp = nested_dict.setdefault(tokens[0], {})
             for token in tokens[1:-1]:
-                tmp = tmp.setdefault(token, dict())
+                tmp = tmp.setdefault(token, {})
             tmp[tokens[-1]] = value
         else:
             nested_dict[tokens[0]] = value
@@ -450,9 +473,8 @@ def _nested_dicts_to_dotted_keys(t, encode=_encode_tree, key=None):
     if isinstance(t, Mapping):
         if t:
             for k in t:
-                k_ = k if key is None else '.'.join((key, k))
-                for k__, v in _nested_dicts_to_dotted_keys(t[k], encode=encode, key=k_):
-                    yield k__, v
+                k_ = k if key is None else ".".join((key, k))
+                yield from _nested_dicts_to_dotted_keys(t[k], encode=encode, key=k_)
         elif key is not None:
             yield key, t
     else:
