@@ -5,6 +5,10 @@
 
 from .synceddict import _SyncedDict
 
+"""
+THIS MODULE IS DEPRECATED!
+"""
+
 
 class SyncedAttrDict(_SyncedDict):
     """A synced dictionary where (nested) values can be accessed as attributes.
@@ -17,13 +21,14 @@ class SyncedAttrDict(_SyncedDict):
         ad = SyncedAttrDict(nested_dict)
         assert ad.a.b == 0
     """
-    _PROTECTED_KEYS = ('_data', '_suspend_sync_', '_load', '_save')
+
+    _PROTECTED_KEYS = ("_data", "_suspend_sync_", "_load", "_save")
 
     def __getattr__(self, name):
         try:
-            return super(SyncedAttrDict, self).__getattribute__(name)
+            return super().__getattribute__(name)
         except AttributeError:
-            if name.startswith('__'):
+            if name.startswith("__"):
                 raise
             try:
                 return self.__getitem__(name)
@@ -32,17 +37,17 @@ class SyncedAttrDict(_SyncedDict):
 
     def __setattr__(self, key, value):
         try:
-            super(SyncedAttrDict, self).__getattribute__('_data')
+            super().__getattribute__("_data")
         except AttributeError:
-            super(SyncedAttrDict, self).__setattr__(key, value)
+            super().__setattr__(key, value)
         else:
-            if key.startswith('__') or key in self.__getattribute__('_PROTECTED_KEYS'):
-                super(SyncedAttrDict, self).__setattr__(key, value)
+            if key.startswith("__") or key in self.__getattribute__("_PROTECTED_KEYS"):
+                super().__setattr__(key, value)
             else:
                 self.__setitem__(key, value)
 
     def __delattr__(self, key):
-        if key.startswith('__') or key in self._PROTECTED_KEYS:
-            super(SyncedAttrDict, self).__delattr__(key)
+        if key.startswith("__") or key in self._PROTECTED_KEYS:
+            super().__delattr__(key)
         else:
             self.__delitem__(key)
