@@ -2,27 +2,28 @@
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
 """Data store implementation with backend HDF5 file."""
+import array
+import errno
 import logging
 import os
-import errno
 import warnings
-import array
+from collections.abc import Mapping, MutableMapping
 from threading import RLock
-from collections.abc import Mapping
-from collections.abc import MutableMapping
 
 from ..errors import InvalidKeyError
 from .dict_manager import DictManager
 
-
 __all__ = [
-    'H5Store', 'H5Group', 'H5StoreManager',
-    'H5StoreClosedError', 'H5StoreAlreadyOpenError',
-    ]
+    "H5Store",
+    "H5Group",
+    "H5StoreManager",
+    "H5StoreClosedError",
+    "H5StoreAlreadyOpenError",
+]
 
 
 def _group_is_pandas_type(group):
-    return 'pandas_type' in group.attrs
+    return "pandas_type" in group.attrs
 
 
 _is_pandas_type = None
@@ -40,9 +41,12 @@ def _load_pandas():
 
             def _is_pandas_type(value):
                 return isinstance(value, pandas.core.generic.PandasObject)
+
         except ImportError:
+
             def _is_pandas_type(value):
-                return False    # Must be False when pandas is not available.
+                return False  # Must be False when pandas is not available.
+
         else:
             _pandas = pandas
 
@@ -52,7 +56,8 @@ def _requires_tables():
         import tables  # noqa
     except ImportError:
         raise ImportError(
-            "Storing and loading pandas objects requires the PyTables package.")
+            "Storing and loading pandas objects requires the PyTables package."
+        )
 
 
 logger = logging.getLogger(__name__)
@@ -73,8 +78,9 @@ def _h5set(store, grp, key, value, path=None):
     handles None values.
     """
     import h5py
-    import numpy    # h5py depends on numpy, so this is safe.
-    path = path + '/' + key if path else key
+    import numpy  # h5py depends on numpy, so this is safe.
+
+    path = path + "/" + key if path else key
 
     # Guard against assigning a group to itself, e.g., `h5s[key] = h5s[key]`,
     # where h5s[key] is a mapping. This is necessary, because the original
@@ -98,7 +104,7 @@ def _h5set(store, grp, key, value, path=None):
 
     # Regular built-in types:
     elif value is None:
-        grp.create_dataset(key, data=None, shape=None, dtype='f')
+        grp.create_dataset(key, data=None, shape=None, dtype="f")
     elif isinstance(value, (int, float, str, bool, array.array)):
         grp[key] = value
     elif isinstance(value, bytes):
@@ -114,23 +120,26 @@ def _h5set(store, grp, key, value, path=None):
 
     # Other types
     else:
-        _load_pandas()   # might be a pandas type
+        _load_pandas()  # might be a pandas type
         if _is_pandas_type(value):
             _requires_tables()
             store.close()
-            with _pandas.HDFStore(store._filename, mode='a') as store_:
+            with _pandas.HDFStore(store._filename, mode="a") as store_:
                 store_[path] = value
             store.open()
         else:
             grp[key] = value
             warnings.warn(
                 "Storage for object of type '{}' appears to have succeeded, but this "
-                "type is not officially supported!".format(type(value)))
+                "type is not officially supported!".format(type(value))
+            )
 
 
 def _h5get(store, grp, key, path=None):
     """Retrieve the underlying data for a key from its h5py container."""
-    path = path + '/' + key if path else key
+    import h5py
+
+    path = path + "/" + key if path else key
     result = grp[key]
 
     if _group_is_pandas_type(result):
@@ -143,7 +152,7 @@ def _h5get(store, grp, key, path=None):
         # Then we re-open the store.
         filename = grp.file.filename
         store.close()
-        with _pandas.HDFStore(filename, mode='r') as store_:
+        with _pandas.HDFStore(filename, mode="r") as store_:
             data = store_[path]
         store.open()
         return data
@@ -153,6 +162,13 @@ def _h5get(store, grp, key, path=None):
             return None
         elif shape:
             return result
+        elif (
+            h5py.version.version_tuple.major >= 3
+            and h5py.check_dtype(vlen=result.dtype) is str
+        ):
+            # h5py >=3.0.0 returns strings as bytes. This returns str for
+            # consistency with past behavior in signac.
+            return result.asstr()[()]
         else:
             return result[()]
     except AttributeError:
@@ -162,9 +178,9 @@ def _h5get(store, grp, key, path=None):
             return result
 
 
-class _ensure_open(object):
+class _ensure_open:
 
-    __slots__ = ['file', 'open', 'kwargs']
+    __slots__ = ["file", "open", "kwargs"]
 
     def __init__(self, file, **kwargs):
         self.file = file
@@ -173,7 +189,7 @@ class _ensure_open(object):
 
     def __enter__(self):
         if self.file._file is None:
-            self.file._open(** self.kwargs)
+            self.file._open(**self.kwargs)
             self.open = True
 
     def __exit__(self, exception_type, exception_value, exception_traceback):
@@ -185,15 +201,16 @@ class _ensure_open(object):
 class H5Group(MutableMapping):
     """An abstraction layer over h5py's Group objects, to manage and return data."""
 
-    __slots__ = ['_store', '_path']
+    __slots__ = ["_store", "_path"]
 
     def __init__(self, store, path):
         self._store = store
         self._path = path
 
     def __repr__(self):
-        return '{}(store={}, path={})'.format(
-            type(self).__name__, repr(self._store), repr(self._path))
+        return "{}(store={}, path={})".format(
+            type(self).__name__, repr(self._store), repr(self._path)
+        )
 
     @property
     def _group(self):
@@ -205,7 +222,13 @@ class H5Group(MutableMapping):
 
     def __setitem__(self, key, value):
         with _ensure_open(self._store):
-            _h5set(self._store, self._group, self._store._validate_key(key), value, self._path)
+            _h5set(
+                self._store,
+                self._group,
+                self._store._validate_key(key),
+                value,
+                self._path,
+            )
             return value
 
     def __delitem__(self, key):
@@ -220,14 +243,14 @@ class H5Group(MutableMapping):
                 return getattr(self._group, name)
 
     def __setattr__(self, key, value):
-        if key.startswith('__') or key in self.__slots__:
-            super(H5Group, self).__setattr__(key, value)
+        if key.startswith("__") or key in self.__slots__:
+            super().__setattr__(key, value)
         else:
             self.__setitem__(key, value)
 
     def setdefault(self, key, value):
         """Set a value for a key if that key is not already set."""
-        super(H5Group, self).setdefault(key, value)
+        super().setdefault(key, value)
         return self.__getitem__(key)
 
     def __iter__(self):
@@ -241,11 +264,11 @@ class H5Group(MutableMapping):
     def __eq__(self, other):
         with _ensure_open(self._store):
             if isinstance(self, Mapping) and isinstance(other, Mapping):
-                return super(H5Group, self).__eq__(other)
+                return super().__eq__(other)
             elif type(other) == type(self):
                 return self._group == other._group
             else:
-                return super(H5Group, self).__eq__(other)
+                return super().__eq__(other)
 
 
 class H5Store(MutableMapping):
@@ -297,14 +320,14 @@ class H5Store(MutableMapping):
         information.
 
     """
-    __slots__ = ['_filename', '_file', '_kwargs']
+    __slots__ = ["_filename", "_file", "_kwargs"]
 
     _thread_lock = RLock()
 
     def __init__(self, filename, **kwargs):
         if not (isinstance(filename, str) and len(filename) > 0):
-            raise ValueError('H5Store filename must be a non-empty string.')
-        self._filename = os.path.realpath(filename)
+            raise ValueError("H5Store filename must be a non-empty string.")
+        self._filename = os.path.abspath(filename)
         self._file = None
         self._kwargs = kwargs
 
@@ -314,10 +337,14 @@ class H5Store(MutableMapping):
         return self._filename
 
     def __repr__(self):
-        return "{}(filename={})".format(type(self).__name__, repr(os.path.relpath(self._filename)))
+        return "{}(filename={})".format(
+            type(self).__name__, repr(os.path.relpath(self._filename))
+        )
 
     def __str__(self):
-        return "{}(filename={})".format(type(self).__name__, repr(os.path.basename(self._filename)))
+        return "{}(filename={})".format(
+            type(self).__name__, repr(os.path.basename(self._filename))
+        )
 
     def __del__(self):
         self.close()
@@ -336,13 +363,14 @@ class H5Store(MutableMapping):
         if self._file is not None:
             raise H5StoreAlreadyOpenError(self)
         import h5py
+
         # We use the default file parameters, which can optionally be overridden
         # by additional keyword arguments (kwargs). This option is intentionally
         # not exposed to the public API.
         parameters = dict(self._kwargs)
         parameters.update(kwargs)
-        if parameters.get('mode', None) is None:
-            parameters['mode'] = 'a'
+        if parameters.get("mode", None) is None:
+            parameters["mode"] = "a"
 
         self._thread_lock.acquire()
         try:
@@ -361,7 +389,7 @@ class H5Store(MutableMapping):
             This H5Store instance.
         """
         if mode is None:
-            mode = self._kwargs.get('mode', 'a')
+            mode = self._kwargs.get("mode", "a")
         return self._open(mode=mode)
 
     def close(self):
@@ -377,7 +405,7 @@ class H5Store(MutableMapping):
                 try:
                     self._thread_lock.release()
                 except RuntimeError as error:
-                    if 'cannot release un-acquired lock' not in str(error):
+                    if "cannot release un-acquired lock" not in str(error):
                         raise
 
     @property
@@ -416,14 +444,14 @@ class H5Store(MutableMapping):
             self._file.flush()
 
     def __getitem__(self, key):
-        key = key if key.startswith('/') else '/' + key
+        key = key if key.startswith("/") else "/" + key
         with _ensure_open(self):
             return _h5get(self, self._file, key)
 
     @staticmethod
     def _validate_key(key):
         """Emit a warning or raise an exception if key is invalid. Returns key."""
-        if '.' in key:
+        if "." in key:
             raise InvalidKeyError("Keys for the H5Store may not contain dots ('.').")
         return key
 
@@ -438,9 +466,9 @@ class H5Store(MutableMapping):
 
     def __getattr__(self, name):
         try:
-            return super(H5Store, self).__getattribute__(name)
+            return super().__getattribute__(name)
         except AttributeError:
-            if name.startswith('__') or name in self.__slots__:
+            if name.startswith("__") or name in self.__slots__:
                 raise
             try:
                 return self.__getitem__(name)
@@ -448,20 +476,20 @@ class H5Store(MutableMapping):
                 raise AttributeError(e)
 
     def __setattr__(self, key, value):
-        if key.startswith('__') or key in self.__slots__:
-            super(H5Store, self).__setattr__(key, value)
+        if key.startswith("__") or key in self.__slots__:
+            super().__setattr__(key, value)
         else:
             self.__setitem__(key, value)
 
     def __delattr__(self, key):
-        if key.startswith('__') or key in self.__slots__:
-            super(H5Store, self).__delattr__(key)
+        if key.startswith("__") or key in self.__slots__:
+            super().__delattr__(key)
         else:
             self.__delitem__(key)
 
     def setdefault(self, key, value):
         """Set a value for a key if that key is not already set."""
-        super(H5Store, self).setdefault(key, value)
+        super().setdefault(key, value)
         return self.__getitem__(key)
 
     def __iter__(self):
@@ -470,21 +498,21 @@ class H5Store(MutableMapping):
 
     def __len__(self):
         try:
-            with _ensure_open(self, mode='r'):
+            with _ensure_open(self, mode="r"):
                 return len(self._file)
-        except (OSError, IOError) as error:
-            if 'errno = {}'.format(errno.ENOENT) in str(error):
-                return 0     # file does not exist
+        except OSError as error:
+            if f"errno = {errno.ENOENT}" in str(error):
+                return 0  # file does not exist
             else:
                 raise
 
     def __contains__(self, key):
         try:
-            with _ensure_open(self, mode='r'):
+            with _ensure_open(self, mode="r"):
                 return key in self._file
-        except (OSError, IOError) as error:
-            if 'errno = {}'.format(errno.ENOENT) in str(error):
-                return False     # file does not exist
+        except OSError as error:
+            if f"errno = {errno.ENOENT}" in str(error):
+                return False  # file does not exist
             else:
                 raise
 
@@ -515,12 +543,15 @@ class H5StoreManager(DictManager):
     :param prefix:
         The directory prefix shared by all stores managed by this class.
     """
+
     cls = H5Store  # type: ignore
-    suffix = '.h5'  # type: ignore
+    suffix = ".h5"  # type: ignore
 
     @staticmethod
     def _validate_key(key):
         """Emit a warning or raise an exception if key is invalid. Returns key."""
-        if '.' in key:
-            raise InvalidKeyError("Keys for the H5StoreManager may not contain dots ('.').")
+        if "." in key:
+            raise InvalidKeyError(
+                "Keys for the H5StoreManager may not contain dots ('.')."
+            )
         return key
