@@ -3,19 +3,29 @@
 # This software is licensed under the BSD 3-Clause License.
 """Wrapper around json parsing library."""
 import logging
-from json import load, loads, JSONEncoder
+from json import JSONEncoder, load, loads
 from json.decoder import JSONDecodeError
 from typing import Any, Dict, Optional
 
+from deprecation import deprecated
+
+from ..version import __version__
+
 logger = logging.getLogger(__name__)
 
 try:
     import numpy
+
     NUMPY = True
 except ImportError:
     NUMPY = False
 
+"""
+THIS MODULE IS DEPRECATED!
+"""
 
+
+# this class is deprecated
 class CustomJSONEncoder(JSONEncoder):
     """Attempt to JSON-encode objects beyond the default supported types.
 
@@ -23,6 +33,13 @@ class CustomJSONEncoder(JSONEncoder):
     an object that is otherwise not serializable, by calling the object's
     `_as_dict()` method.
     """
+
+    @deprecated(
+        deprecated_in="1.7",
+        removed_in="2.0",
+        current_version=__version__,
+        details="The core.json module bundled with signac is deprecated.",
+    )
     def default(self, o: Any) -> Dict[str, Any]:
         if NUMPY:
             if isinstance(o, numpy.number):
@@ -34,12 +51,18 @@ class CustomJSONEncoder(JSONEncoder):
         except AttributeError:
             # Call the super method, which raises a TypeError if it cannot
             # encode the object.
-            return super(CustomJSONEncoder, self).default(o)
+            return super().default(o)
 
 
+@deprecated(
+    deprecated_in="1.7",
+    removed_in="2.0",
+    current_version=__version__,
+    details="The core.json module bundled with signac is deprecated.",
+)
 def dumps(o: Any, sort_keys: bool = False, indent: Optional[int] = None) -> str:
     """Convert a JSON-compatible mapping into a string."""
     return CustomJSONEncoder(sort_keys=sort_keys, indent=indent).encode(o)
 
 
-__all__ = ['loads', 'load', 'dumps', 'JSONDecodeError']
+__all__ = ["loads", "load", "dumps", "JSONDecodeError"]
