@@ -3,8 +3,7 @@
 # This software is licensed under the BSD 3-Clause License.
 """Compute diffs of state points."""
 
-from .contrib.utility import _dotted_dict_to_nested_dicts
-from .contrib.utility import _nested_dicts_to_dotted_keys
+from .contrib.utility import _dotted_dict_to_nested_dicts, _nested_dicts_to_dotted_keys
 
 
 def diff_jobs(*jobs):
@@ -15,6 +14,8 @@ def diff_jobs(*jobs):
     jobs' state points. The comparison is performed over the combined set of
     keys and values.
 
+    See :ref:`signac diff <signac-cli-diff>` for the command line equivalent.
+
     Parameters
     ----------
     \*jobs : sequence[:class:`~signac.contrib.job.Job`]
@@ -52,15 +53,15 @@ def diff_jobs(*jobs):
     if len(jobs) == 0:
         return {}
     else:
-        sps = {}
+        statepoints = {}
         for job in jobs:
-            sps[job] = set(_nested_dicts_to_dotted_keys(job.sp()))
+            statepoints[job] = set(_nested_dicts_to_dotted_keys(job.statepoint()))
 
-        intersection = set.intersection(*sps.values())
+        intersection = set.intersection(*statepoints.values())
 
         diffs = {}
         for job in jobs:
-            unique_sps = sps[job]-intersection
-            diffs[job.id] = _dotted_dict_to_nested_dicts(dict(unique_sps))
+            unique_statepoints = statepoints[job] - intersection
+            diffs[job.id] = _dotted_dict_to_nested_dicts(dict(unique_statepoints))
 
         return diffs
